Receiver interface for the fischertechnik IR control set

Dependencies:   NeedfulThings

Receiver interface for the fischertechnik IR control set.

An mbed port of the code found on this ft community WIKI page The ft IR remote control uses some kind of RC-MM Protocol . When any of the controls is applied the remote control sends updates at a rate of at 1/120ms or 1/90ms depending on the senders frequency setting. Each message delivers the complete remote control status encoded into 30 bits. The structure of the message can be seen in FtControlSetMessage.h.

I assume that the ft control set works fine with standard 38kHz IR detectors. I have used a CHQ0038 from a broken DVD Player. It can be connected directly to the mbed: GND, VCC->3.3V and the signal line to any of the numbered mbed gpios.

The PDM timing of the ft IR control set isn't that exact. Thus receive errors occur quite frequently. I am observing an error rate somewhere between 3% and 5%. This driver "ignores" up to 3 consecutive receive errors, i.e. it just indicates an error but still provides the last correctly received message, before it resets the message to zero after the fourth error.

Revision:
3:038f86bac6cc
Parent:
2:1ae8bb468515
Child:
4:c1517188f6ec
--- a/FtControlSetReceiver.h	Mon Mar 18 22:00:53 2013 +0000
+++ b/FtControlSetReceiver.h	Wed Mar 20 21:55:20 2013 +0000
@@ -12,57 +12,64 @@
 ///
 /// An mbed port of the code found on this <A HREF="http://www.ftcommunity.de/wiki.php?action=show&topic_id=36">ft community WIKI page</a>
 /// The ft IR remote control uses some kind of <A HREF="http://www.sbprojects.com/knowledge/ir/rcmm.php"> RC-MM Protocol </A>.
 /// When any of the controls is applied the remote control sends updates at a rate of 1/90ms~12Hz.
 /// Each message delivers the complete remote control status encoded into 30 bits.
 /// The structure of the message is can be seen in FtControlSetMessage.h.
 /// The protocol uses Pulse Distance Modulation (PDM) with two bits (dibit) per pulse, hence four different
 /// distances. A transmission consists of 16 pulses (15 distances) for a total of 30 bits.
 ///
 /// I assume that the ft control set works fine with standard 38kHz IR detectors. I have used a CHQ0038 from a broken DVD Player.
 /// It can be onnected directly to the mebed: GND, VCC->3.3V and the signal line to any of the numbered mbed gpios.
+///
+/// The PDM timing of the ft IR control set isn't that exact. Thus receive errors occur quite frequently. I am observing an error rate somewhere between
+/// 3% and 5%. This driver "ignores" up to 3 consecutive receive errors, i.e. it just indicates an error but still provides 
+/// the last correctly received message, before it resets the message to zero after the fourth error. Some fluorescent energy saving lamp
+/// might disturb the transmision quite heavily. 
+
+
 class FtControlSetReceiver
 {
     InterruptIn m_pulseIRQ;
 
-    uint32_t m_rxBuffer;
-    uint32_t m_nPulses;
-    uint32_t m_timeOfLastEdge;
-    uint8_t  m_nOnes;
+    uint32_t m_rxBuffer;  /// receive message buffer
+    uint32_t m_nPulses;   /// receive pulse counter (15 pulses per message)  
+    uint32_t m_timeOfLastEdge; /// signal edge time stamp memory 
+    uint8_t  m_nOnes;     /// number of received high bits for parity calculation
     
-    uint32_t m_errorCount;
-    uint8_t  m_nGracefullyIgnoredErrors;  
+    uint32_t m_errorCount;    /// counter of observed receive errors 
+    uint8_t  m_nGracefullyIgnoredErrors;  /// error counter that reset on successful message reception 
 
-    FtControlSetMessage m_lastMessage;
+    FtControlSetMessage m_lastMessage; /// last message received correctly
 
-    FunctionPointer m_callBack;
+    FunctionPointer m_callBack; ///callback for user code that is called afetr successful message reception
 
-    Timeout m_messageTimeout;
-    Timeout m_receiveTimeout;
+    Timeout m_messageTimeout; /// Timeout for message reset when sender is inactive
+    Timeout m_receiveTimeout; /// receive timeout
+     
     
-    //static const uint32_t c_pulseLengthLimits[5];
-    static const uint32_t c_pulseLengthDelta = 100;//101;
-    static const uint32_t c_pulseLengthOffset = 795;//793;
-    static const uint32_t c_maxPulseTime = c_pulseLengthOffset+7*c_pulseLengthDelta/2;
-    static const uint32_t c_messageTimeout[2];
-    static const uint8_t c_errorsToBeGracefullyIgnored = 3;
+    //static const uint32_t c_pulseWidthLimits[5]; /// di-bit discrimination times 
+    static const uint32_t c_pulseWidthDelta = 100;//101;  /// di-bit discrimination pulse width delta
+    static const uint32_t c_pulseWidthOffset = 795;//793; /// di-bit common pulse width base
+    static const uint32_t c_maxPulseTime = c_pulseWidthOffset+7*c_pulseWidthDelta/2;  /// reveive timeout
+    static const uint32_t c_messageTimeout[2]; /// message timeout depends on selected sender frequency
+    static const uint8_t  c_errorsToBeGracefullyIgnored = 3; /// number of consecutive errors after which the message is reset    
+    static const uint32_t c_recoverTimeout = c_maxPulseTime; /// receiver is deactivated for this time after a receive error (IRQ burst suppression) 
     
-    void pulseISR();
-    void messageTimeoutISR();
-    void recoverISR();
+    void pulseISR(); /// pulse handler
+    void messageTimeoutISR(); /// message time out handler
+    void recoverISR();  /// reactivates the pulse handler after recover timeout
 
     void messageReceived(FtControlSetMessage::Status status);
-    void handleReceiveError();
+    void handleReceiveError(); 
     
 public:
     
-    uint32_t msgCnt;
-    
     /// Create a receiver
     /// @parameter in: The pin the IR detector's message line is connected to
     FtControlSetReceiver(PinName in);
 
     /// get the last received message
-    FtControlSetMessage getLastMessage()const {
+    FtControlSetMessage getMessage()const {
         return m_lastMessage;
     };
@@ -12,57 +12,64 @@
 /// An mbed port of the code found on this <A HREF="http://www.ftcommunity.de/wiki.php?action=show&topic_id=36">ft community WIKI page</a>
 /// The ft IR remote control uses some kind of <A HREF="http://www.sbprojects.com/knowledge/ir/rcmm.php"> RC-MM Protocol </A>.
-/// When any of the controls is applied the remote control sends updates at a rate of 1/90ms~12Hz.
+/// When any of the controls is applied the remote control sends updates at a rate of at 1/120ms or 1/90ms depending on the senders frequency setting.
 /// Each message delivers the complete remote control status encoded into 30 bits.
@@ -12,57 +12,64 @@
 /// Each message delivers the complete remote control status encoded into 30 bits.
-/// The structure of the message is can be seen in FtControlSetMessage.h.
-/// The protocol uses Pulse Distance Modulation (PDM) with two bits (dibit) per pulse, hence four different
+/// The structure of the message can be seen in FtControlSetMessage.h.
+/// The protocol uses Pulse Distance Modulation (PDM) with two bits (di-bit) per pulse, hence four different
 /// distances. A transmission consists of 16 pulses (15 distances) for a total of 30 bits.
 ///